------------------------------------------------------------------------
-- Binds
------------------------------------------------------------------------

local ADDON_NAME, NPG = ...

local min = _G.math.min
local wipe = _G.wipe
local tinsert = _G.tinsert
local tremove = _G.tremove
local ipairs = _G.ipairs
local pairs = _G.pairs
local WorldFrame = _G.WorldFrame
local UnitGUID = _G.UnitGUID
local SetFrameRef = NPG._addon.SetFrameRef

NPG.Binds = {}
do
   local BindHandler      = CreateFrame("Frame",          nil, nil, "SecureHandlerAttributeTemplate")
   local BindActionButton = CreateFrame("Button", "NPGAction", nil, "SecureActionButtonTemplate,SecureHandlerBaseTemplate")
   local KeyList = {}
   local _, PLAYER_CLASS = UnitClass("player")
   
   local function SetBindMacro(id, macro)
      local btn = BindActionButton
      
      btn:SetAttribute("type-b"..id, "macro")
      btn:SetAttribute("type-b"..id.."t", "macro")
      btn:SetAttribute("macrotext-b"..id, macro)
      btn:SetAttribute("macrotext-b"..id.."t", macro:gsub("@mouseover","@target"))
   end
   
   local function SetBindSpell(id, spell)
      local btn = BindActionButton
      if tonumber(spell) then spell = GetSpellInfo(spell) end
      
      btn:SetAttribute("type-b"..id, "spell")
      btn:SetAttribute("spell-b"..id, spell)
   end
   
   local function ClearBinds()
      local btn = BindActionButton
      
      for id = 1, #KeyList do
         btn:SetAttribute("type-b"..id, nil)
         btn:SetAttribute("type-b"..id.."t", nil)
         btn:SetAttribute("macrotext-b"..id, nil)
         btn:SetAttribute("macrotext-b"..id.."t", nil)
         btn:SetAttribute("spell-b"..id, nil)
      end
      
      wipe(KeyList)
   end
   
   local function LoadBind(key, bind)
      tinsert(KeyList, key)
      local id = #KeyList
      
      if bind:match("^rt[1-8]") then -- special command for raid target icons
         local rid = bind:sub(3,4)
         SetBindMacro(id, "/npgrt [@mouseover] " .. rid)
      elseif bind:sub(1,1) == "/" then -- if first character is '/', we'll assume the user wants to bind a macro.
         SetBindMacro(id, bind)
      else
         SetBindSpell(id, bind)
      end
   end
   
   SLASH_NPGRAIDTARGET1 = "/npgrt"
   SlashCmdList.NPGRAIDTARGET = function(m)
      local id, unit = SecureCmdOptionParse(m)
      unit = unit or "target"
      id = tonumber(id)
      
      if GetRaidTargetIndex(unit) ~= id then
         SetRaidTarget(unit, id)
      end
   end
   
   local function AssignBinds()
      BindHandler:Execute("wipe(KeyList)")
      for id, key in ipairs(KeyList) do
         BindHandler:Execute(("KeyList[%s] = '%s'"):format(id, key))
      end
   end
   
   local function SetupBindButton()
      local btn = BindActionButton
      
      btn:SetAttribute("checkselfcast", false)
      btn:SetAttribute("checkfocuscast", false)
      
      SecureHandlerUnwrapScript(btn, "OnClick")
      SecureHandlerWrapScript(btn, "OnClick", btn, [[
      local target = core:RunAttribute("_gettarget")
      
      if target == "target" then
         self:SetAttribute("unit", "target")
         
         if self:GetAttribute("type-"..button) == "macro" then
            return button.."t"
         end
         return button
         
      elseif target == "mouseover" then
         self:SetAttribute("unit", "mouseover")
         return button
         
      end
      
      return false
      ]])
      
      btn:SetAttribute("type", "spell")
   end
   
   local function SetupBindHandler()
      SetFrameRef(BindHandler, "actionbtn", BindActionButton)
      
      BindHandler:Execute("KeyList = newtable()")    
      
      BindHandler:SetAttribute("_enablebinds", [[
      for id, key in ipairs(KeyList) do
         self:SetBindingClick(1, key, actionbtn, "b"..id)
      end
      ]])
      
      BindHandler:SetAttribute("_disablebinds", [[
      self:ClearBindings()
      ]])
   end
   
   function NPG.Binds:Setup()
      SetupBindHandler()
      SetupBindButton()
   end
   
   function NPG.Binds:GetHandler()
      return BindHandler
   end
   
   function NPG.Binds:GetButton()
      return BindActionButton
   end
   
   local tmp = {}   
   function NPG.Binds:LoadBinds(bindTable)
      if InCombatLockdown() then
         return 
      end
      
      ClearBinds()
      
      if bindTable and bindTable.all then
         for key, bind in pairs(bindTable.all) do
            LoadBind(key, bind)
         end
      end
      
      if bindTable and bindTable[PLAYER_CLASS] then
         for key, bind in pairs(bindTable[PLAYER_CLASS]) do
            LoadBind(key, bind)
         end
      end
      
      AssignBinds()
   end
end